@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2020 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -83,7 +83,8 @@ unsigned int SDLTest_Random(SDLTest_RandomContext * rndContext)
 
   if (rndContext==NULL) return -1;
 
-  xh = rndContext->x >> 16, xl = rndContext->x & 65535;
+  xh = rndContext->x >> 16;
+  xl = rndContext->x & 65535;
   rndContext->x = rndContext->x * rndContext->a + rndContext->c;
   rndContext->c =
     xh * rndContext->ah + ((xh * rndContext->al) >> 16) +
