@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2020 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -48,6 +48,18 @@ static void openslES_ResumeDevices(void) {}
 static void openslES_PauseDevices(void) {}
 #endif
 
+#if !SDL_AUDIO_DISABLED && SDL_AUDIO_DRIVER_AAUDIO
+extern void aaudio_ResumeDevices(void);
+extern void aaudio_PauseDevices(void);
+SDL_bool aaudio_DetectBrokenPlayState( void );
+#else
+static void aaudio_ResumeDevices(void) {}
+static void aaudio_PauseDevices(void) {}
+static SDL_bool aaudio_DetectBrokenPlayState( void ) { return SDL_FALSE; }
+#endif
+
+
+
 /* Number of 'type' events in the event queue */
 static int
 SDL_NumberOfEvents(Uint32 type)
@@ -110,6 +122,7 @@ Android_PumpEvents_Blocking(_THIS)
 
         ANDROIDAUDIO_PauseDevices();
         openslES_PauseDevices();
+        aaudio_PauseDevices();
 
         if (SDL_SemWait(Android_ResumeSem) == 0) {
 
@@ -122,6 +135,7 @@ Android_PumpEvents_Blocking(_THIS)
 
             ANDROIDAUDIO_ResumeDevices();
             openslES_ResumeDevices();
+            aaudio_ResumeDevices();
 
             /* Restore the GL Context from here, as this operation is thread dependent */
             if (!isContextExternal && !SDL_HasEvent(SDL_QUIT)) {
@@ -156,6 +170,11 @@ Android_PumpEvents_Blocking(_THIS)
             }
         }
     }
+
+    if ( aaudio_DetectBrokenPlayState() ) {
+        aaudio_PauseDevices();
+        aaudio_ResumeDevices();
+    }
 }
 
 void
@@ -178,6 +197,7 @@ Android_PumpEvents_NonBlocking(_THIS)
             if (videodata->pauseAudio) {
                 ANDROIDAUDIO_PauseDevices();
                 openslES_PauseDevices();
+                aaudio_PauseDevices();
             }
 
             backup_context = 0;
@@ -196,6 +216,7 @@ Android_PumpEvents_NonBlocking(_THIS)
             if (videodata->pauseAudio) {
                 ANDROIDAUDIO_ResumeDevices();
                 openslES_ResumeDevices();
+                aaudio_ResumeDevices();
             }
 
             /* Restore the GL Context from here, as this operation is thread dependent */
@@ -232,6 +253,11 @@ Android_PumpEvents_NonBlocking(_THIS)
             }
         }
     }
+
+    if ( aaudio_DetectBrokenPlayState() ) {
+        aaudio_PauseDevices();
+        aaudio_ResumeDevices();
+    }
 }
 
 #endif /* SDL_VIDEO_DRIVER_ANDROID */
